@@ -36,19 +36,24 @@
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Get the metadata columns of interest
 ###
+
 ### Expected metadata columns for GRanges in GFF3 format:
 ###   - required: type, ID
 ###   - optional: Parent, Name, Dbxref, geneID
-###
+### Used in R/makeTxDbFromGFF.R
+GFF3_COLNAMES <- c("type", "ID", "Parent", "Name", "Dbxref", "geneID")
+
 ### Expected metadata columns for GRanges in GTF format:
 ###   - required: type, gene_id, transcript_id
 ###   - optional: exon_id
-###
+### Used in R/makeTxDbFromGFF.R
+GTF_COLNAMES <- c("type", "gene_id", "transcript_id", "exon_id")
 
-.GENE_TYPES <- c("gene", "pseudogene")
-.TX_TYPES <- c("mRNA", "transcript", "primary_transcript",
-               "ncRNA", "rRNA", "snoRNA", "snRNA", "tRNA", "tmRNA")
-.EXON_TYPES <- "exon"
+.GENE_TYPES <- c("gene", "pseudogene", "transposable_element_gene")
+.TX_TYPES <- c("transcript", "pseudogenic_transcript", "primary_transcript",
+               "mRNA", "ncRNA", "rRNA", "snoRNA", "snRNA", "tRNA", "tmRNA",
+               "miRNA", "miRNA_primary_transcript")
+.EXON_TYPES <- c("exon", "pseudogenic_exon")
 .CDS_TYPES <- "CDS"
 .STOP_CODON_TYPES <- "stop_codon"
 
@@ -73,6 +78,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     factor(type, levels=levels_in_use)
 }
 
+### Return a character vector or NULL.
 .get_gene_id <- function(gr_mcols)
 {
     gene_id <- gr_mcols$gene_id
@@ -83,9 +89,16 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     gene_id
 }
 
-.get_transcript_id <- function(gr_mcols)
+.no_id <- function(id) {is.null(id) || all(is.na(id))}
+
+### Return a character vector, or integer vector (if inferred ids), or NULL.
+.get_transcript_id <- function(gr_mcols, gene_id, type)
 {
     transcript_id <- gr_mcols$transcript_id
+    ## We've seen silly GTF files that contain only lines of type transcript
+    ## but no transcript_id tag.
+    if (!.no_id(gene_id) && .no_id(transcript_id) && all(type %in% .TX_TYPES))
+        return(seq_along(type))  # inferred ids
     if (is.null(transcript_id))
         return(NULL)
     if (!is.character(transcript_id))
@@ -93,11 +106,11 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     transcript_id
 }
 
-### If we have "gene_id" and "transcript_id" metadata columns then we assume
-### the GRanges object is in GTF format. Otherwise we assume it's in GFF3
-### format.
+### If we have "gene_id" and "transcript_id" metadata columns (and if they
+### don't contain only NAs) then we assume the GRanges object is in GTF format.
+### Otherwise we assume it's in GFF3 format.
 .is_gtf_format <- function(gene_id, transcript_id)
-    !(is.null(gene_id) || is.null(transcript_id))
+    !(.no_id(gene_id) || .no_id(transcript_id))
 
 .get_ID <- function(gr_mcols, type, gene_id, transcript_id, gtf.format=FALSE)
 {
@@ -162,7 +175,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     substr(ID, start=nchar(prefix)+1L, stop=nchar(ID))
 }
 
-.get_Name <- function(gr_mcols, type, ID)
+.get_Name <- function(gr_mcols, type, ID, transcript_id)
 {
     Name <- gr_mcols$Name
     if (is.null(Name))
@@ -174,6 +187,9 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     ## has a Name (i.e. Name is NA for all the features of that type), then we
     ## infer their Name from their ID.
     for (type_level in levels(type)) {
+        ## We make an exception for transcripts with inferred ids.
+        if ((type_level %in% .TX_TYPES) && is.integer(transcript_id))
+            next
         idx <- which(type == type_level)
         if (all(is.na(Name[idx])))
             Name[idx] <- .infer_Name_from_ID(ID[idx], type_level)
@@ -267,6 +283,21 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     sort(c(which(type %in% .TX_TYPES), gene_as_tx_IDX))
 }
 
+### noextx :== transcript with no exons
+.get_noextx_IDX <- function(tx_IDX, ID, exon_IDX, Parent)
+{
+    tx_IDX[!(ID[tx_IDX] %in% unlist(Parent[exon_IDX], use.names=FALSE))]
+}
+
+### CDS in transcripts with no exons will also be considered exons (i.e. added
+### to the set of exons).
+.get_cds_with_noextx_parent_IDX <- function(cds_IDX, Parent, noextx_IDX, ID)
+{
+    ## as.logical() will fail if a CDS has at least 2 parents and 1 is a
+    ## transcript with no exons and the other is not.
+    cds_IDX[as.logical(unique(Parent[cds_IDX] %in% ID[noextx_IDX]))]
+}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Management of rejected transcripts
@@ -312,6 +343,15 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 ### Extract the 'exons', 'cds', and 'stop_codons' data frames
 ###
 
+.prepare_dropped_msg <- function(dropped, what)
+{
+    msg <- c("The following ", what, " were dropped")
+    if (nrow(dropped) > 6L)
+        msg <- c(msg, " (showing only the 6 first)")
+    c(msg, ":\n", paste(capture.output(print(head(dropped, n=6L))),
+                                       collapse="\n"))
+}
+
 .extract_rank_from_id <- function(id, parent_id)
 {
     if (length(id) == 0L)
@@ -389,6 +429,15 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         is_orphan <- !(tx_id %in% ID)
         norphan <- sum(is_orphan)
         if (norphan != 0L) {
+            dropped <- data.frame(seqid=exon_chrom[is_orphan],
+                                  start=exon_start[is_orphan],
+                                  end=exon_end[is_orphan],
+                                  strand=exon_strand[is_orphan],
+                                  ID=exon_id[is_orphan],
+                                  Parent=tx_id[is_orphan],
+                                  Name=exon_name[is_orphan],
+                                  stringsAsFactors=FALSE)
+            warning(.prepare_dropped_msg(dropped, paste("orphan", what)))
             keep_idx <- which(!is_orphan)
             tx_id <- tx_id[keep_idx]
             exon_id <- exon_id[keep_idx]
@@ -397,7 +446,6 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
             exon_strand <- exon_strand[keep_idx]
             exon_start <- exon_start[keep_idx]
             exon_end <- exon_end[keep_idx]
-            warning(wmsg(norphan, " orphan ", what, "s were dropped"))
         }
     }
 
@@ -424,6 +472,32 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
     exons
 }
 
+### For each transcript with no exons, infer a single exon that is the same as
+### the transcript itself. Other heuristics might have been used earlier to
+### give exons to these poor transcripts with no exons (like inferring the
+### exons from the CDS, to support the GeneDB case) and .add_missing_exons()
+### should only be used as the last and desperate solution to give them an
+### exon after the other heuristics have failed to give them one.
+.add_missing_exons <- function(exons, transcripts)
+{
+    is_noextx <- !(transcripts$tx_id %in% exons$tx_id)
+    missing_exons <- transcripts[is_noextx, , drop=FALSE]
+    if (nrow(missing_exons) == 0L)
+        return(exons)
+    missing_exons$tx_type <- NULL
+    TX2EXON_COLNAMES <- c(
+        tx_id="tx_id",
+        tx_name="exon_name",
+        tx_chrom="exon_chrom",
+        tx_strand="exon_strand",
+        tx_start="exon_start",
+        tx_end="exon_end"
+    )
+    colnames(missing_exons) <- TX2EXON_COLNAMES[colnames(missing_exons)]
+    missing_exons$exon_rank <- 1L
+    rbind(exons, missing_exons)
+}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Extract the 'transcripts' data frame
@@ -509,6 +583,15 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         warning(wmsg("The following transcripts have multiple parts that ",
                      "were merged: ", in1string))
     }
+    ## We've seen silly GFF/GTF files where each transcript in the file is
+    ## reported to be on its own contig and spans it (start=1) but no strand
+    ## is reported for the transcript. We set the strand to "+" for all these
+    ## transcripts.
+    if (all(transcripts$tx_strand == "*")
+     && anyDuplicated(transcripts$tx_chrom) == 0L
+     && all(transcripts$tx_start == 1L)) {
+        transcripts$tx_strand <- rep.int(strand("+"), nrow(transcripts))
+    }
     transcripts
 }
 
@@ -744,7 +827,7 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
 ### makeTxDbFromGRanges()
 ###
 
-.normarg_metadata <- function(metadata, Genome)
+.normarg_metadata <- function(metadata, Genome, taxonomyId)
 {
     if (!is.null(metadata)) {
         if (!is.data.frame(metadata))
@@ -756,13 +839,29 @@ GFF_FEATURE_TYPES <- c(.GENE_TYPES, .TX_TYPES, .EXON_TYPES,
         df1 <- data.frame(name="Genome", value=Genome, stringsAsFactors=FALSE)
         metadata <- rbind(metadata, df1)
     }
+    if (!is.na(taxonomyId)) {
+        GenomeInfoDb:::.checkForAValidTaxonomyId(taxonomyId)
+        org <- GenomeInfoDb:::.lookupSpeciesFromTaxId(taxonomyId)
+        organism <- paste(org[1,2], org[1,3])
+        idx <- match("Organism", metadata$name)
+        if (!is.na(idx))
+            metadata <- metadata[-idx, , drop=FALSE]
+        idx <- match("Taxonomy ID", metadata$name)
+        if (!is.na(idx))
+            metadata <- metadata[-idx, , drop=FALSE]
+        df2 <- data.frame(name=c("Organism", "Taxonomy ID"),
+                          value=c(organism, taxonomyId),
+                          stringsAsFactors=FALSE)
+        metadata <- rbind(metadata, df2)
+    }
     metadata
 }
 
 ### If 'drop.stop.codons' is TRUE then the "stop_codon" lines are ignored.
 ### Otherwise (the default) the stop codons are considered to be part of the
 ### CDS and merged to them.
-makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL)
+makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL,
+                                taxonomyId=NA)
 {
     if (!is(gr, "GenomicRanges"))
         stop("'gr' must be a GRanges object")
@@ -777,20 +876,20 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL)
     if (!isTRUEorFALSE(drop.stop.codons))
         stop("'drop.stop.codons' must be TRUE or FALSE")
 
-    metadata <- .normarg_metadata(metadata, Genome)
+    metadata <- .normarg_metadata(metadata, Genome, taxonomyId)
 
     gr_mcols <- mcols(gr)
 
     ## Get the metadata columns of interest.
     type <- .get_type(gr_mcols)
     gene_id <- .get_gene_id(gr_mcols)
-    transcript_id <- .get_transcript_id(gr_mcols)
+    transcript_id <- .get_transcript_id(gr_mcols, gene_id, type)
     gtf.format <- .is_gtf_format(gene_id, transcript_id)
     ID <- .get_ID(gr_mcols, type, gene_id, transcript_id,
                   gtf.format=gtf.format)
     Parent <- .get_Parent(gr_mcols, type, gene_id, transcript_id,
                           gtf.format=gtf.format)
-    Name <- .get_Name(gr_mcols, type, ID)
+    Name <- .get_Name(gr_mcols, type, ID, transcript_id)
 
     ## Get the gene, cds, stop_codon, exon, and transcript indices.
     gene_IDX <- .get_gene_IDX(type)
@@ -808,6 +907,21 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL)
                                           exon_with_gene_parent_IDX, Parent)
     tx_IDX <- .get_tx_IDX(type, gene_as_tx_IDX)
 
+    noextx_IDX <- .get_noextx_IDX(tx_IDX, ID, exon_IDX, Parent)
+    if (length(noextx_IDX) != 0L) {
+        ## Infer exons for transcripts with no exons (noextx).
+
+        ## For a noextx with CDS (the GeneDB case): infer 1 exon per CDS that
+        ## is the same as the CDS itself.
+        cds_with_noextx_parent_IDX <- .get_cds_with_noextx_parent_IDX(
+                                              cds_IDX, Parent,
+                                              noextx_IDX, ID)
+        exon_IDX <- sort(c(exon_IDX, cds_with_noextx_parent_IDX))
+
+        ## For a noextx with no CDS (the miRBase case): we'll take care of
+        ## them later with .add_missing_exons().
+    }
+
     ## Extract the 'exons', 'cds', 'stop_codons', 'transcripts',
     ## and 'genes' data frames.
     exons <- .extract_exons_from_GRanges(exon_IDX, gr, ID, Name, Parent,
@@ -861,19 +975,25 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL)
             in1string <- paste0(sort(drop2_tx), collapse=", ")
             warning(wmsg(
                 "The following transcripts were dropped because no genomic ",
-                "ranges could be found for them and they have exons on more ",
-                "than one chromosome so their ranges could not be inferred: ",
-                in1string))
+                "ranges could be found for them and their ranges could not ",
+                "be inferred from their exons either (because they have them ",
+                "on more than one chromosome): ", in1string))
         }
         if (length(drop3_tx) != 0L) {
             in1string <- paste0(sort(drop3_tx), collapse=", ")
             warning(wmsg(
                 "The following transcripts were dropped because no genomic ",
-                "ranges could be found for them and they have exons on both ",
-                "strands so their ranges could not be inferred: ", in1string))
+                "ranges could be found for them and their ranges could not ",
+                "be inferred from their exons either (because they have them ",
+                "on both strands): ", in1string))
         }
     }
 
+    ## We might still have transcripts with no exons (noextx). Take care of
+    ## them now.
+    exons <- .add_missing_exons(exons, transcripts)
+
+    ## .make_splicings() can also reject some transcripts.
     .flush_rejected_tx_envir()
     splicings <- .make_splicings(exons, cds, stop_codons)
     drop_tx <- .get_rejected_transcripts()
@@ -884,7 +1004,8 @@ makeTxDbFromGRanges <- function(gr, drop.stop.codons=FALSE, metadata=NULL)
     }
 
     ## Turn the "tx_id" column in 'splicings', 'genes', and 'transcripts' into
-    ## an integer vector. TODO: Maybe makeTxDb() could take care of this.
+    ## an integer vector.
+    ## TODO: Maybe makeTxDb() could take care of that.
     splicings_tx_id <- match(splicings$tx_id, transcripts$tx_id)
     if (any(is.na(splicings_tx_id)))
         stop(wmsg("some exons are linked to transcripts ",
@@ -953,7 +1074,8 @@ test_makeTxDbFromGRanges_on_Ensembl_organism_gtf <- function(organism)
     download.file(url, local_gtf_file)
 
     cat("Import ", local_gtf_file, " as GRanges object 'gr' ... ", sep="")
-    gr <- import(local_gtf_file)
+    gr <- import(local_gtf_file, colnames=GTF_COLNAMES,
+                                 feature.type=GFF_FEATURE_TYPES)
     cat("\n")
 
     cat("txdb1 <- makeTxDbFromGRanges(gr) ... ", sep="")
@@ -1028,27 +1150,32 @@ library(rtracklayer)
 GFF3_files <- system.file("extdata", "GFF3_files", package="GenomicFeatures")
 
 file1 <- file.path(GFF3_files, "TheCanonicalGene_v1.gff3")
-gr1 <- import(file1, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr1 <- import(file1, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb1 <- makeTxDbFromGRanges(gr1)
 txdb1
 
 file2 <- file.path(GFF3_files, "TheCanonicalGene_v2.gff3")
-gr2 <- import(file2, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr2 <- import(file2, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb2 <- makeTxDbFromGRanges(gr2)
 txdb2
 
 file3 <- file.path(GFF3_files, "a.gff3")
-gr3 <- import(file3, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr3 <- import(file3, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb3 <- makeTxDbFromGRanges(gr3)
 txdb3
 
 file4 <- file.path(GFF3_files, "dmel-1000-r5.11.filtered.gff")
-gr4 <- import(file4, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr4 <- import(file4, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb4 <- makeTxDbFromGRanges(gr4)
 txdb4  # exactly the same as with makeTxDbFromGFF()
 
 file5 <- file.path(GFF3_files, "NC_011025.gff")
-gr5 <- import(file5, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr5 <- import(file5, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb5 <- makeTxDbFromGRanges(gr5)
 txdb5
 
@@ -1057,7 +1184,8 @@ filename <- paste0(genome, "_genomic.gff.gz")
 url <- paste0("ftp://ftp.ncbi.nlm.nih.gov/genomes/all/", genome, "/", filename)
 file6 <- file.path(tempdir(), file6)
 download.file(url, file6)
-gr6 <- import(file6, format="gff3", feature.type=GFF_FEATURE_TYPES)
+gr6 <- import(file6, format="gff3", colnames=GFF3_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 
 ## Compared with makeTxDbFromGFF():
 ##
@@ -1076,12 +1204,14 @@ GTF_files <- system.file("extdata", "GTF_files", package="GenomicFeatures")
 ## test1.gtf grabbed from http://mblab.wustl.edu/GTF22.html (5 exon gene with
 ## 3 translated exons).
 file1 <- file.path(GTF_files, "test1.gtf")
-gr1 <- import(file1, format="gtf", feature.type=GFF_FEATURE_TYPES)
+gr1 <- import(file1, format="gtf", colnames=GTF_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb1 <- makeTxDbFromGRanges(gr1)
 txdb1
 
 file2 <- file.path(GTF_files, "Aedes_aegypti.partial.gtf")
-gr2 <- import(file2, format="gtf", feature.type=GFF_FEATURE_TYPES)
+gr2 <- import(file2, format="gtf", colnames=GTF_COLNAMES,
+                     feature.type=GFF_FEATURE_TYPES)
 txdb2 <- makeTxDbFromGRanges(gr2)
 txdb2
 }
