@@ -452,7 +452,7 @@ supportedUCSCtables <- function()
         query <- ucscTableQuery(session, table=tablename)
         ucsc_table <- getTable(query)
         message("OK")
-        if (!all(hasCol(ucsc_table, c(Lcolname, Rcolname))))
+        if (!all(has_col(ucsc_table, c(Lcolname, Rcolname))))
             stop("expected cols \"", Lcolname, "\" or/and \"",
                  Rcolname, "\" not found in table ", tablename)
         Lcol <- ucsc_table[[Lcolname]]
@@ -683,7 +683,7 @@ supportedUCSCtables <- function()
 ### Download and preprocess the 'chrominfo' data frame.
 ###
 
-.makeUCSCChrominfo <- function(genome, circ_seqs,
+.makeUCSCChrominfo <- function(genome, circ_seqs=DEFAULT_CIRC_SEQS,
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath")
 {
     message("Download and preprocess the 'chrominfo' data frame ... ",
@@ -699,25 +699,23 @@ supportedUCSCtables <- function()
     chrominfo <- data.frame(
         chrom=ucsc_chrominfotable$chrom,
         length=ucsc_chrominfotable$size,
-        is_circular=matchCircularity(ucsc_chrominfotable$chrom, circ_seqs)
+        is_circular=make_circ_flags_from_circ_seqs(ucsc_chrominfotable$chrom,
+                                                   circ_seqs)
     )
-    chrom_ids <- rankSeqlevels(chrominfo$chrom)
-    chrominfo <- chrominfo[order(chrom_ids), ]
+    oo <- order(rankSeqlevels(chrominfo[ , "chrom"]))
+    chrominfo <- chrominfo[oo, , drop=FALSE]
+    rownames(chrominfo) <- NULL
     message("OK")
     chrominfo
 }
 
-
-### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Allow users to discover 'chrominfo' data frame.
-###
-
+## User-friendly wrapper to .makeUCSCChrominfo().
 getChromInfoFromUCSC <- function(genome,
           goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath")
 {
-  chromInfo <-.makeUCSCChrominfo(genome, circ_seqs=character(),
+  chrominfo <-.makeUCSCChrominfo(genome, circ_seqs=character(),
                                  goldenPath_url=goldenPath_url)
-  chromInfo[,1:2]
+  chrominfo[ , 1:2]
 }
 
 
@@ -727,17 +725,23 @@ getChromInfoFromUCSC <- function(genome,
 ###
 
 .prepareUCSCMetadata <- function(genome, tablename, gene_id_type, full_dataset,
-                                 miRBaseBuild=NA)
+                                 taxonomyId=NA, miRBaseBuild=NA)
 {
     message("Prepare the 'metadata' data frame ... ",
             appendLF=FALSE)
     if (!isSingleStringOrNA(miRBaseBuild))
         stop("'miRBaseBuild' must be a a single string or NA")
+    if(is.na(taxonomyId)){
+        taxonomyId <- GenomeInfoDb:::.taxonomyId(UCSCGenomeToOrganism(genome))
+    }else{
+        GenomeInfoDb:::.checkForAValidTaxonomyId(taxonomyId)
+    }
+        
     metadata <- data.frame(
-        name=c("Data source", "Genome", "Organism", "UCSC Table",
+        name=c("Data source", "Genome", "Organism", "Taxonomy ID", "UCSC Table",
                "Resource URL", "Type of Gene ID", "Full dataset",
                "miRBase build ID"),
-        value=c("UCSC", genome, UCSCGenomeToOrganism(genome),
+        value=c("UCSC", genome, UCSCGenomeToOrganism(genome), taxonomyId,
                 tablename, "http://genome.ucsc.edu/", gene_id_type,
                 ifelse(full_dataset, "yes", "no"), miRBaseBuild)
     )
@@ -753,8 +757,9 @@ getChromInfoFromUCSC <- function(genome,
 .makeTxDbFromUCSCTxTable <- function(ucsc_txtable, genes,
         genome, tablename, gene_id_type,
         full_dataset,
-        circ_seqs,
+        circ_seqs=DEFAULT_CIRC_SEQS,
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath",
+        taxonomyId=NA,
         miRBaseBuild=NA)
 {
     ucsc_txtable <- setDataFrameColClass(ucsc_txtable, .UCSC_TXCOL2CLASS,
@@ -775,7 +780,7 @@ getChromInfoFromUCSC <- function(genome,
     genes <- .makeUCSCGenes(genes, ucsc_txtable)
     chrominfo <- .makeUCSCChrominfo(genome, circ_seqs, goldenPath_url)
     metadata <- .prepareUCSCMetadata(genome, tablename, gene_id_type,
-                                     full_dataset, miRBaseBuild)
+                                     full_dataset, taxonomyId,  miRBaseBuild)
 
     message("Make the TxDb object ... ", appendLF=FALSE)
     txdb <- makeTxDb(transcripts, splicings, genes=genes,
@@ -799,6 +804,7 @@ makeTxDbFromUCSC <- function(genome="hg19",
         circ_seqs=DEFAULT_CIRC_SEQS,
         url="http://genome.ucsc.edu/cgi-bin/",
         goldenPath_url="http://hgdownload.cse.ucsc.edu/goldenPath",
+        taxonomyId=NA,
         miRBaseBuild=NA)
 {
     track <- .tablename2track(tablename, genome)
@@ -849,19 +855,15 @@ makeTxDbFromUCSC <- function(genome="hg19",
     } else {
         stop("GenomicFeatures internal error: invalid 'mapdef'")
     }
-
     .makeTxDbFromUCSCTxTable(ucsc_txtable, txname2geneid$genes,
                              genome, tablename,
                              txname2geneid$gene_id_type,
                              full_dataset=is.null(transcript_ids),
                              circ_seqs=circ_seqs,
                              goldenPath_url=goldenPath_url,
+                             taxonomyId=taxonomyId,
                              miRBaseBuild=miRBaseBuild)
 }
 
-makeTranscriptDbFromUCSC <- function(...)
-{
-    .Deprecated("makeTxDbFromUCSC")
-    makeTxDbFromUCSC(...)
-}
+makeTranscriptDbFromUCSC <- function(...) .Defunct("makeTxDbFromUCSC")
 
