@@ -1,7 +1,7 @@
 #####################################################################
 ## Helpers to access/process the table names and columns
 .getTableColMapping <- function(x){
-  conn <- AnnotationDbi:::dbconn(x)
+  conn <- dbconn(x)
   tables <- DBI::dbListTables(conn)
   tCols <- sapply(tables, DBI::dbListFields, conn=conn)
   ## right up front we are getting rid of metadata and chrominfo tables...
@@ -182,23 +182,24 @@
   paste(sqlCol, "IN (", chrs,")")
 }
 
-.select <- function(x, keys, columns, keytype){
+.select <- function(x, keys, columns, keytype, ...){
+  extraArgs <- list(...)
   if(missing(keys)){stop("'keys' must be a character vector")}
   if(missing(columns)){stop("'columns' must be a character vector")}
   ## Some argument checking
-  AnnotationDbi:::.testSelectArgs(x, keys=keys, cols=columns, keytype=keytype)
+  if('skipValidKeysTest' %in% names(extraArgs)){
+      skipValidKeysTest<-extraArgs[["skipValidKeysTest"]]}else{
+          skipValidKeysTest<-FALSE}
+  AnnotationDbi:::.testSelectArgs(x, keys=keys, cols=columns,
+                                  keytype=keytype,
+                                  skipValidKeysTest=skipValidKeysTest)
+  
   ## 1st we check the keytype to see if it is valid:
   if(is.na(keys(x, keytype)[1]) & length(keys(x, keytype))==1){ 
     stop(paste("There do not appear to be any keys",
                "for the keytype you have specified."))
     }
 
-  ## keys should NOT be NAs, but if they are, warn and then filter them.
-  if(any(is.na(keys))) {
-    warning(paste("NA keys have been dropped"))
-    keys <- keys[!is.na(keys)]
-  }
-
   ## we used to add TXID to cnames, which forces splicing to always be included
   ## Splicing is a almost always needed, but almost never requested.
   ## cnames <- unique(c(columns, "TXID", keytype))
@@ -243,7 +244,7 @@
                  majorJoin)
   }
   
-  res <- AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x), sql)
+  res <- AnnotationDbi:::dbQuery(dbconn(x), sql)
 
   if(length(keys) > 1000){ ##Then drop the extras now(in event there are some)
       ktColId <- .reverseColAbbreviations(x, keytype)
@@ -275,30 +276,30 @@
 }
 
 
-##  Remove this select warning function after 2.13 has released
-.selectWarnTxDb <- function(x, keys, columns, keytype, ...){
+## ##  Remove this select warning function after 2.13 has released
+## .selectWarnTxDb <- function(x, keys, columns, keytype, ...){
     
-    ## remove condition after 2.13
-    extraArgs <- list(...)
-    if("cols" %in% names(extraArgs)){
-        ## warn the user about the old argument
-        AnnotationDbi:::.colsArgumentWarning()
-        ## ## then call it using cols in place of columns    
-        ## if(missing(keytype)){
-        ##     .select(x, keys, columns=extraArgs[["cols"]], keytype = columns)
-        ## }else{
-        ##     .select(x, keys, columns=extraArgs[["cols"]], keytype = keytype)
-        ## }        
-    }else{
-        .select(x, keys, columns, keytype)
-    }
-}
+##     ## remove condition after 2.13
+##     extraArgs <- list(...)
+##     if("cols" %in% names(extraArgs)){
+##         ## warn the user about the old argument
+##         AnnotationDbi:::.colsArgumentWarning()
+##         ## ## then call it using cols in place of columns    
+##         ## if(missing(keytype)){
+##         ##     .select(x, keys, columns=extraArgs[["cols"]], keytype = columns)
+##         ## }else{
+##         ##     .select(x, keys, columns=extraArgs[["cols"]], keytype = keytype)
+##         ## }        
+##     }else{
+##         .select(x, keys, columns, keytype)
+##     }
+## }
 
 
 setMethod("select", "TxDb",
     function(x, keys, columns, keytype, ...) {
-          .selectWarnTxDb(x, keys, columns, keytype, ...)
-##           .select(x, keys, columns, keytype)
+##          .selectWarnTxDb(x, keys, columns, keytype, ...)
+           .select(x, keys, columns, keytype, ...)
         }
 )
 
@@ -326,19 +327,19 @@ setMethod("columns", "TxDb",
 .keys <- function(x, keytype){
     AnnotationDbi:::.testForValidKeytype(x, keytype)
     res <- switch(EXPR = keytype,
-             "GENEID" = AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "GENEID" = AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT gene_id FROM gene", 1L),
-             "TXID" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "TXID" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT _tx_id FROM transcript", 1L),
-             "TXNAME" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "TXNAME" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT tx_name FROM transcript", 1L),
-             "EXONID" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "EXONID" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT _exon_id FROM exon", 1L),
-             "EXONNAME" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "EXONNAME" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT exon_name FROM exon", 1L),
-             "CDSID" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "CDSID" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT _cds_id FROM cds", 1L),                  
-             "CDSNAME" =  AnnotationDbi:::dbQuery(AnnotationDbi:::dbconn(x),
+             "CDSNAME" =  AnnotationDbi:::dbQuery(dbconn(x),
                "SELECT DISTINCT cds_name FROM cds", 1L),                  
                   stop(paste(keytype, "is not a supported keytype.",
                              " Please use the keytypes",
