@@ -17,8 +17,23 @@
     ans
 }
 
+## helper to translate back to what is expected from seqinfo()
+.translateToSeqInfo <- function(txdb, x){
+    tr <- load_chrominfo(txdb, set.col.class=TRUE)$chrom[txdb$user2seqlevels0]
+    names(tr) <- txdb$user_seqlevels
+    idx <- match(x, tr)
+    names(tr)[idx]
+}
+
 ## !!!
 ## TODO: I think that this helper is screwing things up
+.baseNamedActiveSeqs <- function(txdb){
+    trueNames <- load_chrominfo(txdb, set.col.class=TRUE)$chrom
+    actSqs <- .isActiveSeq(txdb)
+    names(actSqs) <- trueNames[txdb$user2seqlevels0] ## limit result to these.
+    actSqs
+}
+
 .getOnlyActiveSeqs <- function(txdb){
     actSqs <- .baseNamedActiveSeqs(txdb)
     names(actSqs)[actSqs]
@@ -118,6 +133,79 @@
     .assignMetadataList(ans, txdb)
 }
 
+### 'columns' must be a named character vector with the names being used to
+### rename the columns.
+.split_df_into_GRL <- function(txdb, df, columns, by, use.names)
+{
+    rownames(df) <- NULL
+    f <- df[[1L]]
+    gr <- makeGRangesFromDataFrame(df[-1L], seqinfo=get_TxDb_seqinfo0(txdb))
+    gr_mcols <- setNames(df[columns], names(columns))
+    mcols(gr) <- DataFrame(gr_mcols, check.names=FALSE)
+    grl <- split(gr, f)
+    grl <- keep_user_seqlevels_from_TxDb(grl, txdb)
+    grl <- .set.group.names(grl, use.names, txdb, by)
+    .assignMetadataList(grl, txdb)
+}
+
+.extract_exons_or_cds_by <- function(txdb, proxy_table, by=c("tx", "gene"),
+                                     use.names=FALSE)
+{
+    proxy_table_columns <- TXDB_table_columns(proxy_table)
+    proxy_column <- proxy_table_columns[["id"]]
+
+    if (by == "tx") {
+        ## Group by transcript.
+        orderby <- c("_tx_id", "exon_rank")
+        ans_columns <- c(id=proxy_column,
+                         name=proxy_table_columns[["name"]],
+                         "exon_rank")
+
+        ## We use a single SELECT query.
+        columns <- c(orderby, TXDB_table_columns(proxy_table))
+        exons <- TxDb_SELECT_from_splicing_bundle(txdb, columns,
+                                                  orderby=orderby,
+                                                  join_type="INNER")
+    } else {
+        ## Group by gene.
+        orderby <- c("gene_id", proxy_column)
+        ans_columns <- proxy_table_columns[c("id", "name")]
+
+        ## We use 2 SELECT queries and join the results ourselves. This join
+        ## is actually faster than the SQL JOIN.
+
+        ## 1st SELECT query.
+        gene_exons <- TxDb_SELECT_from_INNER_JOIN(txdb, "splicing", orderby,
+                                                  orderby=orderby)
+        if (proxy_table == "cds") {
+            ## Proxy column is from the "splicing" table, not from the "cds"
+            ## table (which was not even involved in the JOIN in the first
+            ## place), so it can contain NAs. Remove these rows.
+            keep_me <- !is.na(gene_exons[[2L]])
+            gene_exons <- gene_exons[keep_me, , drop=FALSE]
+        }
+
+        ## 2nd SELECT query.
+        columns <- TXDB_table_columns(proxy_table)
+        exons <- TxDb_SELECT_from_INNER_JOIN(txdb, proxy_table, columns)
+
+        ## Join the results.
+        gene_id <- gene_exons[ , "gene_id"]
+        gene_id <- factor(gene_id, levels=unique(gene_id))
+        join_idx <- match(gene_exons[ , proxy_column], exons[ , proxy_column])
+        exons <- cbind(gene_id=gene_id, exons[join_idx, ])
+    }
+
+    user_columns <- names(ans_columns)
+    has_noname <- user_columns %in% c(NA_character_, "")
+    prefix_idx <- which(!has_noname)
+    user_columns[prefix_idx] <- paste(proxy_table, user_columns[prefix_idx],
+                                      sep="_")
+    user_columns[has_noname] <- ans_columns[has_noname]
+    names(ans_columns) <- user_columns
+    .split_df_into_GRL(txdb, exons, ans_columns, by, use.names)
+}
+
 setGeneric("transcriptsBy", signature="x",
     function(x, by=c("gene", "exon", "cds"), ...)
         standardGeneric("transcriptsBy")
@@ -157,14 +245,7 @@ setMethod("exonsBy", "TxDb",
     function(x, by=c("tx", "gene"), use.names=FALSE)
     {
         by <- match.arg(by)
-        distinct <- gene_in_join <- by == "gene"
-        order_by_exon_rank <- by == "tx"
-        .featuresBy(x, by, "exon",
-                    distinct=distinct,
-                    splicing_in_join=TRUE,
-                    gene_in_join=gene_in_join,
-                    order_by_exon_rank=order_by_exon_rank,
-                    use.names=use.names)
+        .extract_exons_or_cds_by(x, "exon", by, use.names=use.names)
     }
 )
 
@@ -176,14 +257,7 @@ setMethod("cdsBy", "TxDb",
     function(x, by=c("tx", "gene"), use.names=FALSE)
     {
         by <- match.arg(by)
-        distinct <- gene_in_join <- by == "gene"
-        order_by_exon_rank <- by == "tx"
-        .featuresBy(x, by, "cds",
-                    distinct=distinct,
-                    splicing_in_join=TRUE,
-                    gene_in_join=gene_in_join,
-                    order_by_exon_rank=order_by_exon_rank,
-                    use.names=use.names)
+        .extract_exons_or_cds_by(x, "cds", by, use.names=use.names)
     }
 )
 
@@ -216,14 +290,7 @@ setMethod("intronsByTranscript", "TxDb",
 {
     ans <- load_splicings(txdb)
     ids <- unique(ans$tx_id[!is.na(ans$cds_id)])
-    ans <- ans[ans$tx_id %in% ids, ]
-
-    ## modify results to not respect our activeSeqs mask.
-    isActSeq <- .isActiveSeq(txdb)
-    ## We only care about the ones that are TRUE
-    isActSeq <- isActSeq[isActSeq]
-    ## remove unwanted stuff from df 
-    ans[ans$exon_chrom %in% names(isActSeq),]  
+    ans[ans$tx_id %in% ids, ]
 }
 
 ### 'tx_id': character or integer vector with runs of identical elements (one
@@ -231,7 +298,7 @@ setMethod("intronsByTranscript", "TxDb",
 ### 'exons_with_cds': integer vector containing the indices of the elements
 ### in 'tx_id' corresponding to exons with a CDS. The indices must be sorted
 ### in ascending order.
-### Returns the indices of the first exon with a CDS for every transcript, plus
+### Returns for each transcript the indices of the first exon with a CDS, plus
 ### the indices of the preceding exons in the transcript.
 ### Note that, for each transcript, exons that have a 5' UTR are all the exons
 ### before the first exon with a CDS, including the first exon with a CDS (even
@@ -252,7 +319,7 @@ setMethod("intronsByTranscript", "TxDb",
 }
 
 ### 'tx_id', 'exons_with_cds': same as for .exons_with_5utr().
-### Returns the indices of the last exon with a CDS for every transcript, plus
+### Returns for each transcript the indices of the last exon with a CDS, plus
 ### the indices of the following exons in the transcript.
 ### Note that, for each transcript, exons that have a 3' UTR are all the exons
 ### after the last exon with a CDS, including the last exon with a CDS (even
@@ -271,23 +338,23 @@ setMethod("intronsByTranscript", "TxDb",
     S4Vectors:::fancy_mseq(lengths, offset=offset)
 }
 
-.makeUTRsByTranscript <- function(x, splicings, utr_start, utr_end)
+.makeUTRsByTranscript <- function(txdb, splicings, utr_start, utr_end)
 {
-    activeNames <- names(.isActiveSeq(x))[.isActiveSeq(x)]
-    seqinfo <- seqinfo(x)[activeNames]
+    seqinfo0 <- get_TxDb_seqinfo0(txdb)
     cols <- paste0("exon_", c("id", "name", "rank"))
-    grg <- GRanges(seqnames = factor(
+    gr <- GRanges(seqnames = factor(
                      splicings$exon_chrom,
-                     levels =seqlevels(seqinfo)),
-                   ranges = IRanges(start = utr_start, end = utr_end),
-                   strand = strand(splicings$exon_strand),
-                   splicings[cols],
-                   seqinfo = seqinfo)
-    idx <- width(grg) != 0L  # drop 0-width UTRs
-    split(grg[idx], splicings$tx_id[idx])
+                     levels =seqlevels(seqinfo0)),
+                  ranges = IRanges(start = utr_start, end = utr_end),
+                  strand = strand(splicings$exon_strand),
+                  splicings[cols],
+                  seqinfo = seqinfo0)
+    idx <- width(gr) != 0L  # drop 0-width UTRs
+    grl <- split(gr[idx], splicings$tx_id[idx])
+    keep_user_seqlevels_from_TxDb(grl, txdb)
 }
 
-.make5UTRsByTranscript <- function(x, splicings, use.names=FALSE)
+.make5UTRsByTranscript <- function(txdb, splicings, use.names=FALSE)
 {
     exons_with_cds <- which(!is.na(splicings$cds_id))
     idx <- .exons_with_5utr(splicings$tx_id, exons_with_cds)
@@ -303,13 +370,13 @@ setMethod("intronsByTranscript", "TxDb",
     utr_start[idx] <- splicings$cds_end[idx] + 1L
 
     ## split by grouping variable
-    ans <- .makeUTRsByTranscript(x, splicings, utr_start, utr_end)
-    ans <- .set.group.names(ans, use.names, x, "tx")
-    ans <- .assignMetadataList(ans, x)
+    ans <- .makeUTRsByTranscript(txdb, splicings, utr_start, utr_end)
+    ans <- .set.group.names(ans, use.names, txdb, "tx")
+    ans <- .assignMetadataList(ans, txdb)
     ans
 }
 
-.make3UTRsByTranscript <- function(x, splicings, use.names=FALSE)
+.make3UTRsByTranscript <- function(txdb, splicings, use.names=FALSE)
 {
     exons_with_cds <- which(!is.na(splicings$cds_id))
     idx <- .exons_with_3utr(splicings$tx_id, exons_with_cds)
@@ -325,9 +392,9 @@ setMethod("intronsByTranscript", "TxDb",
     utr_end[idx] <- splicings$cds_start[idx] - 1L
 
     ## split by grouping variable
-    ans <- .makeUTRsByTranscript(x, splicings, utr_start, utr_end)
-    ans <- .set.group.names(ans, use.names, x, "tx")            
-    ans <- .assignMetadataList(ans, x)
+    ans <- .makeUTRsByTranscript(txdb, splicings, utr_start, utr_end)
+    ans <- .set.group.names(ans, use.names, txdb, "tx")            
+    ans <- .assignMetadataList(ans, txdb)
     ans
 }
 
